#ifndef __FILE_MLEUSR_H_SEEN__
#define __FILE_MLEUSR_H_SEEN__

/*-----------------------------------------------------------------------------

Copyright (C) 2013

A. Ronald Gallant
Post Office Box 659
Chapel Hill NC 27514-0659
USA   

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

-----------------------------------------------------------------------------*/
#include "libmle.h"
#include "mle_base.h"

namespace mle {
    
  class elec_usrmod;

  typedef elec_usrmod usrmod_type;

  class elec_usrmod : public libmle::usrmod_base {
  private:
    scl::realmat data;
    scl::realmat rho;
    INTEGER lrho;
    INTEGER lstats;
    scl::realmat a;
    scl::realmat B;
    scl::realmat R;
    scl::realmat yhat;
    scl::realmat ehat;
    scl::realmat zhat;
    void set_parms();
    bool get_obj(scl::realmat& obj);
  public:
    elec_usrmod
      (const scl::realmat& dat, INTEGER len_mod_parm, INTEGER len_mod_func,
       const std::vector<std::string>& mod_pfvec,
       const std::vector<std::string>& mod_alvec, 
       std::ostream& detail);
    INTEGER len_rho() {return lrho;}
    INTEGER len_stats() {return lstats;}
    bool get_stats(scl::realmat& stats);
    void get_rho(scl::realmat& parm) { parm = rho; }
    void set_rho(const scl::realmat& parm) { rho = parm; set_parms();}
    bool support(const scl::realmat& rho);
    scl::den_val prior(const scl::realmat& rho, const scl::realmat& stats);
    scl::den_val likelihood();
    bool get_scores(scl::realmat& scores);
    void write_usrvar(const char* filename) 
    { 
      std::string stem = filename;
      size_t suffix = stem.find(std::string("dat"));
      if (suffix != std::string::npos) stem = stem.substr(0, suffix);
      if (likelihood().positive) {
        std::string fn;
        fn = stem + "yhat.dat"; vecwrite(fn.c_str(), yhat);
        fn = stem + "ehat.dat"; vecwrite(fn.c_str(), ehat);
        fn = stem + "zhat.dat"; vecwrite(fn.c_str(), zhat);
      }
    }
  };
}

#endif
